﻿package com.selenetan.asteroids 
{
    import com.selenetan.core.STGlobal;
    import com.selenetan.core.STGroup;
	import com.selenetan.core.STObject;
    import com.selenetan.core.STWorld;
    import flash.display.Sprite;
    import flash.filters.GlowFilter;
	
	/**
     * ...
     * @author Selene
     */
    public class Asteroid extends STObject
    {
        public static const ASTEROID_SIZES:Array = [16, 24, 32];
        
        public var size:int;
        
        private static const DENSITY:Number = 0.1;     // mass units per pixel^2
        private static const RADIUS_RATIO:Number = 0.5;
        private static const NUM_VERTICES:int = 12;
        private static var DRAW_COMMANDS:Vector.<int> = Vector.<int>([1, 2, 2, 2,  2, 2, 2, 2,  2, 2, 2, 2,  2]);
        
        private var _vertices:Vector.<Number>;
        private var _maxRadius:Number;
        
        // **DEBUG/UTILITY
        private static var _instances:uint = 0;
        private var _id:uint;
        
        public function Asteroid($size:int) 
        {
            var s:Sprite = new Sprite();
            
            // Generate the asteroid shape 
            size = $size;
            s = drawNewAsteroid(s);
            
            super(s, s);
            velocity.x = Math.random() * 100 - 50;
            velocity.y = Math.random() * 100 - 50;
            
            mass = DENSITY * Math.PI * hitRadius * hitRadius;
            
            size = $size;
            
            _id = _instances;
            ++_instances;
            
            health = 5;
        }
        
        
        override public function update():void
        {
            //trace(this.toString() + ".update()");
            
            // Make them wrap around the edge of the world
            var wWidth:Number = STGlobal.currentWorld.worldWidth;
            while (x < 0) {
                x += wWidth;
            }
            while (x > wWidth) {
                x -= wWidth;
            }
            
            super.update();
        }
        
        
        override public function reset($x:Number = 0, $y:Number = 0, $args:Object = null):void
        {
            velocity.x = Math.random() * 100 - 50;
            velocity.y = Math.random() * 100 - 50;
            //super.reset(Math.random() * 800, Math.random() * 400 + 100);
            super.reset($x, $y);
            
            if ($args.size != undefined) {
                size = $args.size;
                drawNewAsteroid(sprite);
            }
        }
        
        
        private function drawNewAsteroid(s:Sprite):Sprite
        {
            s.graphics.clear();
            
            _maxRadius = ASTEROID_SIZES[size];
            var totalRadius:Number = 0;
            var maxRadius:Number = _maxRadius;
            var minRadius:Number = _maxRadius * RADIUS_RATIO;
            var currRadius:Number;
            _vertices = new Vector.<Number>(NUM_VERTICES * 2 + 2, true);
            for (var i:int = 0; i < NUM_VERTICES; ++i) {
                currRadius = Math.random() * (maxRadius - minRadius) + minRadius;
                totalRadius += currRadius;
                _vertices[2 * i] = currRadius * Math.cos(i * 2.0 * Math.PI / NUM_VERTICES);
                _vertices[2 * i + 1] = currRadius * Math.sin(i * 2.0 * Math.PI / NUM_VERTICES);
            }
            _vertices[2 * NUM_VERTICES] = _vertices[0];
            _vertices[2 * NUM_VERTICES + 1] = _vertices[1];
            
            hitRadius = totalRadius / NUM_VERTICES;
            
            s.graphics.lineStyle(1, 0xFFFFFF, 1, true);
            s.graphics.drawPath(DRAW_COMMANDS, _vertices);
            
            return s;
        }
        
        public function toString():String {
            return "[Asteroid #" + _id + "]";
        }
        
        
    }

}