﻿package com.selenetan.core 
{
    import flash.display.Bitmap;
    import flash.display.BitmapData;
    import flash.display.DisplayObject;
    import flash.display.Shape;
    import flash.display.Sprite;
	/**
     * ...
     * @author Selene
     */
    public class STWorld
    {
        public static const WRAPPING_NONE:int = 0;
        public static const WRAPPING_HORIZONTAL:int = 1;
        public static const WRAPPING_VERTICAL:int = 2;
        public static const WRAPPING_BOTH:int = 3;
        
        /**
         * Holds all the sprites for the STObjects that have been added to the world.
         */
        public var container:Sprite;
        public var children:Array;
        
        public var viewWidth:Number;
        public var viewHeight:Number;
        /**
         * X-coordinate of the upper-left corner of the view rectangle
         */
        public var viewX:Number;
        /**
         * Y-coordinate of the upper-left corner of the view rectangle
         */
        public var viewY:Number;
        
        public var worldWidth:Number;
        public var worldHeight:Number;
        
        public var wrapping:int;
        
        /**
         * Used when you have a wrapping map
         */
        public var bgColor:uint;
        
        protected var _follows:STObject;
        protected var _drawContainer:Sprite;
        protected var _drawCanvas:Bitmap;
        
        protected var _fadeRect:Shape;
        
        public function STWorld($container:Sprite, $viewWidth:Number, $viewHeight:Number, $wrapping:int = WRAPPING_NONE) 
        {
            container = $container;
            viewWidth = $viewWidth;
            viewHeight = $viewHeight;
            viewX = 0;
            viewY = 0;
            
            worldWidth = container.width;
            worldHeight = container.height;
            
            wrapping = $wrapping;
            
            if ($wrapping) {
                _drawContainer = new Sprite();
                _drawContainer.addChild(container);
                _drawCanvas = new Bitmap(new BitmapData($viewWidth, $viewHeight), "auto", true);
                _drawCanvas.bitmapData.fillRect(_drawCanvas.bitmapData.rect, bgColor);
                
                _fadeRect = new Shape();
                _fadeRect.graphics.beginFill(bgColor, 0.08);
                _fadeRect.graphics.drawRect(0, 0, $viewWidth, $viewHeight);
            }
            
            children = [];
            _follows = null;
        }
        
        
        /**
         * Adds an STObject to the world and its sprite to the world container.
         * @param	object  The STObject to be added.
         */
        public function add(object:STObject):void
        {
            children.push(object);
            container.addChild(object.sprite);
        }
        
        /**
         * Sets an object for the "camera" to follow
         * @param	object
         */
        public function setFollows(object:STObject):void
        {
            _follows = object;
        }
        
        
        /**
         * Updates motion and logic for this and all children
         */
        public function update():void
        {
            var numChildren:int = children.length;
            for (var i:int = 0; i < numChildren; ++i) {
                STObject(children[i]).update();
            }
        }
        
        /**
         * Updates graphics for this and all children.
         * NOTE: The wrapping render code assumes that no object that is larger than the viewport size will extend past the boundaries of the map.
         *       i.e., Backgrounds exactly the size of the map will be drawn correctly, but giant objects that go past the edges will not.
         */
        public function render():void
        {
            if (_follows) {
                // View coordinates are the upper-left of the rectangle
                viewX = _follows.x - viewWidth * 0.5;
                viewY = _follows.y - viewHeight * 0.5;
            }
            
            
            var numChildren:int = children.length;
            for (var i:int = 0; i < numChildren; ++i) {
                STObject(children[i]).render();
            }
            
            // Draw the wrapping world
            if (wrapping == WRAPPING_NONE) {
                container.x = -viewX;
                container.y = -viewY;
            }
            else {
                _drawCanvas.bitmapData.fillRect(_drawCanvas.bitmapData.rect, bgColor);
                
                var origViewX:Number = viewX;
                var origViewY:Number = viewY;
                
                if (viewX < 0) {
                    viewX = ( worldWidth + viewX) % worldWidth;
                }
                else {
                    viewX = viewX % worldWidth;
                }
                
                if (viewY < 0) {
                    viewY = (worldHeight + viewY) % worldHeight;
                }
                else {
                    viewY = viewY % worldHeight;
                }
                
                // Do we need to draw the sides wrapping?
                if (wrapping & WRAPPING_HORIZONTAL) {
                    if (viewX < viewWidth ) {
                        container.x = -worldWidth - viewX;
                        _drawCanvas.bitmapData.draw(_drawContainer, null, null, null, null, true);
                    }
                    if (viewX > worldWidth - 2.0 * viewWidth) {
                        container.x = worldWidth - viewX;
                        _drawCanvas.bitmapData.draw(_drawContainer, null, null, null, null, true);
                    }
                    container.x = -viewX;
                }
                
                // Do we need to draw the top and bottom wrapping?
                if (wrapping & WRAPPING_VERTICAL) {
                    if (viewY < viewHeight ) {
                        container.y = -worldHeight - viewY;
                        _drawCanvas.bitmapData.draw(_drawContainer, null, null, null, null, true);
                    }
                    if (viewY > worldHeight - 2.0 * viewHeight) {
                        container.y = worldHeight - viewY;
                        _drawCanvas.bitmapData.draw(_drawContainer, null, null, null, null, true);
                    }
                    container.y = -viewY;
                }
                
                // Do we need to draw the corners wrapping?
                if (wrapping & WRAPPING_BOTH) {
                    if (viewX < viewWidth && viewY < viewHeight) {
                        container.x = -worldWidth - viewX;
                        container.y = -worldHeight - viewY;
                        _drawCanvas.bitmapData.draw(_drawContainer, null, null, null, null, true);
                    }
                    
                    if (viewX > worldWidth - 2 * viewWidth && viewY < viewHeight) {
                        container.x = -worldWidth - viewX;
                        container.y = -worldHeight - viewY;
                        _drawCanvas.bitmapData.draw(_drawContainer, null, null, null, null, true);
                    }
                    
                    if (viewX < viewWidth && viewY > worldHeight - 2 * viewHeight) {
                        container.x = -worldWidth - viewX;
                        container.y = worldHeight - viewY;
                        _drawCanvas.bitmapData.draw(_drawContainer, null, null, null, null, true);
                    }
                    
                    if (viewX > worldWidth - 2 * viewWidth && viewY > worldHeight - 2 * viewHeight) {
                        container.x = worldWidth - viewX;
                        container.y = worldHeight - viewY;
                        _drawCanvas.bitmapData.draw(_drawContainer, null, null, null, null, true);
                    }
                    
                    
                    container.x = -viewX;
                    container.y = -viewY;
                    _drawCanvas.bitmapData.draw(_drawContainer, null, null, null, null, true);
                    
                    viewX = origViewX;
                    viewY = origViewY;
                }
                
            }
            
        }
        
        
        /**
         * Returns the DisplayObject that contains the graphical display
         */
        public function get sprite():DisplayObject
        {
            if (wrapping) {
                return _drawCanvas;
            }
            else {
                return container;
            }
        }
    }

}