﻿package com.selenetan.statemachine 
{
	import com.selenetan.statemachine.IStateMachine;
	
	/**
     * ...
     * @author Selene
     */
    public class SimpleStateMachine implements IStateMachine
    {
        
        protected var _states:Object;
        protected var _currState:IState;
        
        /**
         * 
         * @param	$states             An array of IStates to populate the machine with.
         * @param   $startStateName     The name of the first state. If blank, defaults to the first state in the array.
         */
        public function SimpleStateMachine($states:Array, $startStateName:String = "") 
        {
            _states = { };
            for (var i:int = 0; i < $states.length; ++i) {
                _states[$states[i].name] = $states[i];
            }
            
            if ($startStateName == "") {
                _currState = _states[$states[0].name];
            }
            else {
                _currState = _states[$startStateName];
            }
            
            _currState.enter();
        }
        
        /**
         * Attempts to add a state to the machine.
         * @param	state   The state to add
         * @return  Returns true if the state was added, false if the state already exists.
         */
        public function addState(state:IState):Boolean
        {
            if (_states[state.name]) {
                return false;
            }
            
            _states[state.name] = state;
            return true;
        }
        
        
        /**
         * Try to transition to another state.
         * @param	stateName   The name of the state to transition to.
         * @return  Returns true if the transition is successful, false if it is not.
         */
        public function requestTransition(stateName:String):void
        {
            if (!_states[stateName]) {
                trace("No such state " + stateName);
                return;
            }
            
            _states[_currState].exit();
            _currState = _states[stateName];
            _currState.enter();
        }
        
        
        public function getCurrentStateName():String
        {
            return _currState.name;
        }
        
    }

}